# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Greenbone AG

from datetime import datetime
from typing import Iterator

from troubadix.plugin import LinterError, LinterResult


def parse_date(date: str) -> datetime:
    return datetime.strptime(date[:25], "%Y-%m-%d %H:%M:%S %z")


def check_date(
    date: str, date_name: str, file: str, plugin: str
) -> Iterator[LinterResult]:
    """
    Checks if a given date string is correctly formatted.
    Example: "2017-11-29 13:56:41 +0100 (Wed, 29 Nov 2017)"
    """

    if not date:
        yield LinterError(
            f"No {date_name} has been found.",
            file=file,
            plugin=plugin,
        )
        return

    try:
        # 2017-11-29 13:56:41 +0100
        date_left = parse_date(date)

        # Wed, 29 Nov 2017
        date_right = datetime.strptime(date[27:43], "%a, %d %b %Y")

        week_day_parsed = date_right.strftime("%a")

    except ValueError:
        yield LinterError(
            f"Missing or incorrectly formatted {date_name}.",
            file=file,
            plugin=plugin,
        )
        return

    # Position of the 3 char day of the week abbreviation: Mon, Tue, Wed...
    week_day_str = date[27:30]

    if date_left.date() != date_right.date():
        yield LinterError(
            f"The {date_name} consists of two different dates.",
            file=file,
            plugin=plugin,
        )

    # Check correct weekday
    elif week_day_str != week_day_parsed:
        formatted_date = week_day_parsed
        yield LinterError(
            f"Wrong day of week. Please change it from '{week_day_str}"
            f"' to '{formatted_date}'.",
            file=file,
            plugin=plugin,
        )


def compare_date_with_last_modification_date(
    date: str, date_name: str, last_mod_date: str, file: str, plugin: str
) -> Iterator[LinterResult]:

    yield from check_date(
        last_mod_date,
        "last_modification",
        file=file,
        plugin=plugin,
    )

    try:
        if parse_date(date) > parse_date(last_mod_date):
            yield LinterError(
                f"The {date_name} must not be greater than "
                "last_modification date.",
                file=file,
                plugin=plugin,
            )

    except ValueError:
        yield LinterError(
            f"Could not compare {date_name} with last_modification date.",
            file=file,
            plugin=plugin,
        )



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Greenbone AG

from pathlib import Path

from troubadix.plugins.if_statement_syntax import CheckIfStatementSyntax

from . import PluginTestCase


class CheckIfStatementSyntaxTestCase(PluginTestCase):
    """Test cases for the CheckIfStatementSyntax plugin."""

    def test_valid_if_statements(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        if(condition) {
            display("block statement");
        }

        if(another_condition) display("single statement");

        if(nested_condition) {
            if(inner_condition) {
                display("nested");
            }
        }
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_unclosed_parenthesis(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        if(unclosed_condition {
            display("this should fail");
        }
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIn("Unclosed parenthesis", results[0].message)
        self.assertIn("line 2", results[0].message)

    def test_unclosed_brace(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        if(condition) {
            display("missing closing brace");
        # Missing closing brace
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIn("Unclosed brace", results[0].message)

    def test_missing_statement(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        if(condition)
        # No statement follows
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIn("Missing statement", results[0].message)

    def test_semicolon_after_condition(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        if(condition);
        {
            display("this will always execute");
        }
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIn("Semicolon after if condition", results[0].message)

    def test_missing_semicolon_in_expression(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        if(condition) display("missing semicolon")
        # No semicolon at end of expression
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIn("Missing expression", results[0].message)

    def test_comment_handling(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        # This comment has an unclosed if(condition that should be ignored
        if(real_condition) {
            # Another comment with if(fake
            display("real code");
        }
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_complex_nested_error(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = """
        if(outer_condition) {
            if(inner_condition {  # Missing closing parenthesis
                display("nested error");
            }
        }
        """

        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckIfStatementSyntax(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIn("Unclosed parenthesis", results[0].message)



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Greenbone AG


from pathlib import Path
from typing import Iterator

from troubadix.helper.if_block_parser import find_if_statements
from troubadix.helper.remove_comments import remove_comments
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckIfStatementSyntax(FileContentPlugin):
    """Check for syntax errors in if statements.

    This plugin uses the if statement parser to detect common syntax errors such as:
    - Unclosed parentheses in if conditions
    - Unclosed braces in if blocks
    - Missing statements after if conditions
    - Semicolons after if conditions (which make following blocks always execute)
    - Missing semicolons in single-line expressions
    """

    name = "check_if_statement_syntax"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """Check the file content for if statement syntax errors."""
        # Remove comments to avoid false positives from commented code
        comment_free_content = remove_comments(file_content)

        result = find_if_statements(comment_free_content)
        for error in result.errors:
            yield LinterError(
                error.error_type.value.format(line=error.line),
                file=nasl_file,
                plugin=self.name,
            )



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Greenbone AG

from pathlib import Path
from typing import Iterator

from troubadix.helper import ScriptTag, get_script_tag_pattern
from troubadix.helper.date_format import (
    check_date,
    compare_date_with_last_modification_date,
)
from troubadix.plugin import FileContentPlugin, LinterResult


class CheckSeverityDate(FileContentPlugin):
    name = "check_severity_date"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:

        if nasl_file.suffix == ".inc":
            return

        severity_date_pattern = get_script_tag_pattern(ScriptTag.SEVERITY_DATE)
        last_modification_pattern = get_script_tag_pattern(
            ScriptTag.LAST_MODIFICATION
        )

        if not (
            match_severity_date := severity_date_pattern.search(file_content)
        ):
            return

        yield from check_date(
            match_severity_date.group("value"),
            "severity_date",
            nasl_file,
            self.name,
        )

        if match_last_mod_date := last_modification_pattern.search(
            file_content
        ):
            yield from compare_date_with_last_modification_date(
                match_severity_date.group("value"),
                "severity_date",
                match_last_mod_date.group("value"),
                nasl_file,
                self.name,
            )
